/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.editors.browser;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.tools.DefaultTreeChangeListener;
import com.unitesk.requality.eclipse.tools.ITreeStateListener;
import com.unitesk.requality.eclipse.tools.ResourceStorage;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class ReqMarker extends EditorPart {
    // private static final String DEFAULT_BASE_ENCODING = "UTF-8";
    public static final String ID = "com.unitesk.requality.editors.browser.ReqMarker";

    protected ReqDocBrowser browser;

    private Document doc;
    private TreeDB db;
    private Location initialLocation;

    public void openDocument(URL url) {
        String path = url.toString();
        browser.setUrl(path);
    }

    public ReqDocBrowser getBrowser() {
        return browser;
    }

    private ISelectionChangedListener locChangeListener = new ISelectionChangedListener() {
        
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            if(browser!=null && !browser.isDisposed()) {
                IStructuredSelection s = (IStructuredSelection) event.getSelection();
                Location loc = (Location) s.getFirstElement();
                browser.setTargetLocation(loc);
            }
        }
    };
    private ITreeChangeListener treeChangeListener = new DefaultTreeChangeListener() {
        public void deleted(TreeNode node) {
            if (doc.getQualifiedId().equals(node.getQualifiedId())) {
                getSite().getShell().getDisplay().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(ReqMarker.this,
                                false);
                    }
                });
            }
        }

        @Override
        public void changed(TreeNode node) {
            getSite().getShell().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    if (TreesTracker.getTracker().getProject(db) == null) {
                        getEditorSite().getPage().closeEditor(ReqMarker.this,
                                false);
                        return;
                    }
                    browser.updateLocations();
                }
            });
        }
    };

    private ITreeStateListener dbStateListener = new ITreeStateListener() {

        @Override
        public void open(TreeDB db) {
        }

        @Override
        public void close(TreeDB db) {
            getSite().getShell().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    getEditorSite().getPage()
                            .closeEditor(ReqMarker.this, false);
                }
            });
        }
    };

    private ReqMarkerOutlinePage reqMarkerOutlinePage;

    final class BrowserKeyListener implements KeyListener {
        @Override
        public void keyReleased(KeyEvent e) {
            if ((e.stateMask & SWT.CTRL) != 0) {
                if (e.keyCode == 'r')
                    new AddSelectionToRequirement(ReqMarker.this).run();
                if (e.keyCode == 'd')
                    new RemoveLocationFromRequirement(ReqMarker.this).run();
            }
        }

        @Override
        public void keyPressed(KeyEvent e) {
        }
    }

    public ReqMarker() {
    }

    protected void updateTargetRequirement() {
        List<Location> locationList = getTargetRequirement().getLocations();
        if (locationList.size() == 0)
            return;
        Location location = locationList.get(0);

        setTargetLocation(location);
    }

    public void setTargetLocation(Location loc) {
        if (browser != null) {
            browser.setTargetLocation(loc);
        }
    }

    public void saveDocument() throws UnsupportedEncodingException {
        String htmlContent = browser.serializeDocument();

        if (htmlContent != null && htmlContent.length() > 0) {
            try {
                ResourceStorage rs = (ResourceStorage) db.getStorage();
                IFolder folder = rs.getResourceFolder(doc.getUUId());
                IFile f = folder.getFile(doc.getResourceName());
                ByteArrayInputStream in = new ByteArrayInputStream(
                        htmlContent.getBytes("UTF-8"));
                f.setContents(in, false, true, null);
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }
        if(reqMarkerOutlinePage!=null && !reqMarkerOutlinePage.getControl().isDisposed()) {
            reqMarkerOutlinePage.refresh();
        }
    }

    @Override
    public void doSave(IProgressMonitor monitor) {
    }

    @Override
    public void doSaveAs() {
        throw new UnsupportedOperationException("Unexpected method call");
    }

    @Override
    public void init(IEditorSite site, IEditorInput input)
            throws PartInitException {
        try {
            if (input instanceof NodeEditorInput) {
                NodeEditorInput docInput = (NodeEditorInput) input;
                if (!docInput.getStorage().getNode().getType()
                        .equals("Document")
                        && !docInput.getStorage().getNode().getType()
                                .equals("Requirement")) {
                    throw new PartInitException("Wrong input type: "
                            + docInput.getStorage().getNode().getType());
                }
                if (docInput.getStorage().getNode() instanceof Requirement) {
                    Requirement req = (Requirement) docInput.getStorage()
                            .getNode();
                    if (req.getLocations().size() == 0) {
                        // can't open document if requirement has no locations
                        return;
                    }
                    doc = (Document) req.getLocations().get(0).getParent();

                    // // not the best way to go
                    // IEditorReference ref;
                    // if((ref =
                    // Activator.findMarkEditorForDocument(doc.getQualifiedId()))!=null)
                    // {
                    // ReqMarker m = (ReqMarker) ref.getEditor(false);
                    // m.close(f);
                    // }
                    initialLocation = (Location) req.getLocations().get(0);
                } else {
                    doc = (Document) docInput.getStorage().getNode();
                    initialLocation = (Location) docInput.getData();
                }
                db = docInput.getStorage().getNode().getTreeDB();
            } else if (input instanceof FileEditorInput) {
                MessageDialog
                        .openError(getSite().getShell(),
                                "Unsupported editor open method",
                                "not yet implemented");
                // FileEditorInput inputFile = (FileEditorInput) input;
                // IProject project = inputFile.getFile().getProject();
                // db = (TreeDB) TreesTracker.getTracker().getTree(project);
                // String docPath = new File(db.getDocPath()).getAbsolutePath();
                // String docId = inputFile.getFile().getLocation().toOSString()
                // .substring((int) docPath.length());
                // doc = (Document) db.getDocument(docId);
            }
        } catch (CoreException e) {
            e.printStackTrace();
        }
        setInput(input);
        setPartName(doc.getId());
        setSite(site);

        db.addListener(treeChangeListener);
        TreesTracker.getTracker().addListener(dbStateListener);
    }

    @Override
    public boolean isDirty() {
        return false;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }

    @Override
    public void createPartControl(Composite parent) {
        parent.setLayout(new GridLayout(1, false));

        Composite browserPanel = new Composite(parent, SWT.NONE);
        browserPanel.setLayoutData(new GridData(GridData.FILL_BOTH));
        browserPanel.setLayout(new FillLayout());

        browser = new ReqDocBrowser(browserPanel, SWT.NONE);

        browser.getWebBrowser().setMenu(createMenu());

        new BrowserFunction(browser.getWebBrowser(),
                "getLocationsToRequirementMap") {
            @Override
            public Object function(Object[] arguments) {
                if (arguments.length == 0)
                    return null;
                return getLocationsToRequirementMap((Object[]) arguments[0]);
            }
        };

        new BrowserFunction(browser.getWebBrowser(), "setInitialLocation") {
            @Override
            public Object function(Object[] arguments) {
                setTargetLocation(initialLocation);
                return null;
            }
        };

        browser.getWebBrowser().addKeyListener(new BrowserKeyListener());

        // Get path to file

        // TODO: check for resource existing
        String openPath = doc.getHTMLPath();
        try {
            URL url = new File(openPath).toURI().toURL();
            openDocument(url);
        } catch (MalformedURLException e1) {
            e1.printStackTrace();
        }
    }

    public Requirement[] getLocationsToRequirement(String locId) {

        Location loc = (Location) db
                .getNode(doc.getQualifiedId() + "/" + locId);
        return loc.getRequirements();
    }

    protected Object[][] getLocationsToRequirementMap(Object locations[]) {

        Map<String, ArrayList<String>> res = new HashMap<String, ArrayList<String>>();
        ArrayList<String> locQIds = new ArrayList<String>();
        for (Object locationId : locations) {
            locQIds.add(doc.getQualifiedId() + "/" + locationId);
        }

        for (TreeNode node : new DeepFirstTreeWalker(db.getNode(Requirement
                .getTypeRootQId()))) {
            if (!node.getType().equals("Requirement")) {
                continue;
            }
            Requirement req = (Requirement) node;

            for (String locQId : locQIds) {
                if (req.getLocation(locQId) != null) {
                    if (res.get(locQId) == null) {
                        res.put(locQId, new ArrayList<String>());
                    }
                    res.get(locQId).add(req.getName());
                }
            }
        }

        Object[][] result = new Object[locations.length][];

        for (int i = 0; i < res.keySet().size(); i++) {
            List<String> loc2req = new ArrayList<String>();
            loc2req.add((String) locations[i]);
            List<String> reqIds = res.get(doc.getQualifiedId() + "/"
                    + locations[i]);
            if (reqIds != null) {
                loc2req.addAll(reqIds);
            }
            result[i] = loc2req.toArray();
        }
        return result;
    }

    private Menu createMenu() {
        MenuManager menuMgr = new MenuManager("#PopupMenu");
        menuMgr.setRemoveAllWhenShown(true);

        menuMgr.addMenuListener(new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                manager.add(new AddSelectionToRequirement(ReqMarker.this));

                Requirement req = getTargetRequirement();
                if (req != null) {
                    if (req.getQualifiedId().equals(
                            Requirement.getTypeRootQId())) {
                        manager.add(new AddSelectionToRequirement(
                                ReqMarker.this, req, AddSelectionToRequirement
                                        .getNextId(req)));
                    } else {
                        manager.add(new AddSelectionToRequirement(
                                ReqMarker.this, req));

                        manager.add(new AddSelectionToRequirement(
                                ReqMarker.this, req, AddSelectionToRequirement
                                        .getNextId(req)));
                    }
                } else {
                    req = (Requirement) db.getNode(Requirement.getTypeRootQId());
                    manager.add(new AddSelectionToRequirement(ReqMarker.this,
                            req, AddSelectionToRequirement.getNextId(req)));
                }

                manager.add(new RemoveLocationFromRequirement(ReqMarker.this));

                manager.add(new Action("Show identifiers") {
                    @Override
                    public boolean isChecked() {
                        return browser.areIdentifiersVisible();
                    }

                    @Override
                    public int getStyle() {
                        return IAction.AS_CHECK_BOX;
                    }

                    @Override
                    public void run() {
                        browser.toggleIdentifiersVisibility();
                    }
                });

                manager.add(new Action("Show version") {
                    @Override
                    public void run() {
                        browser.showBrowserInfo();
                    }
                });

                manager.add(new Action("Export page...") {
                    @Override
                    public void run() {
                        String htmlContent = browser.serializeDocumentRaw();
                        htmlContent = htmlContent.replace(Activator
                                .getDefault().getJSToolsPath(), "");
                        DirectoryDialog dialog = new DirectoryDialog(getSite()
                                .getShell());
                        String path = dialog.open();
                        if (path != null) {
                            try {
                                copyPageFile(path + File.separator);
                                Writer out = new BufferedWriter(
                                        new OutputStreamWriter(
                                                new FileOutputStream(path
                                                        + File.separator
                                                        + doc.getId()), "UTF8"));
                                out.write(htmlContent);
                                out.close();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                    }

                    private void copyPageFile(String path) throws IOException {
                        String jsPath = Activator.getDefault().getJSToolsPath();
                        copyFile(new File(jsPath + "jquery.js"), new File(path
                                + "jquery.js"));
                        copyFile(new File(jsPath + "edit.js"), new File(path
                                + "edit.js"));
                        copyFile(new File(jsPath + "edit.css"), new File(path
                                + "edit.css"));
                    }
                });

            }
        });

        Menu menu = menuMgr.createContextMenu(browser);
        return menu;
    }

    public static void copyFile(File in, File out) throws IOException {
        FileInputStream fis = new FileInputStream(in);
        FileOutputStream fos = new FileOutputStream(out);
        try {
            byte[] buf = new byte[1024];
            int i = 0;
            while ((i = fis.read(buf)) != -1) {
                fos.write(buf, 0, i);
            }
        } catch (IOException e) {
            throw e;
        } finally {
            if (fis != null)
                fis.close();
            if (fos != null)
                fos.close();
        }
    }

    @Override
    public void setFocus() {
        browser.setFocus();
    }

    public TreeDB getTreeDB() {
        return db;
    }

    public Document getDocument() {
        return doc;
    }

    public boolean addSelectionTo(Requirement req) {
        UUID locId = UUID.randomUUID();
        Location loc = (Location) doc.createChild(locId.toString(), "Location");
        String reqQId = req.getQualifiedId();
        String desc = browser.createLocation(loc.getId(), reqQId);
        loc.setDescription(desc);
        loc.saveAttributes();
        req.addLocation(loc.getQualifiedId());
        req.saveAttributes();
        try {
            saveDocument();
            TreesTracker.getResourceByNode(doc).refreshLocal(
                    IResource.DEPTH_INFINITE, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }

    public void removeLocation(String locationId) {
        removeLocation(locationId, false);
    }

    public void removeLocation(String locationId, boolean forse) {
        for (Requirement req : getLocationsToRequirement(locationId)) {
            String locQId = doc.getQualifiedId() + "/"
                    + locationId;
            Location loc = req.getLocation(locQId);
            if (loc == null)
                break;

            req.removeLocation(locQId);
            if (!forse && req.getLocations().size() == 0
                    && req.sizeChildren() == 0) {
                if (MessageDialog.openQuestion(getSite().getShell(),
                        "Delete requirement", "Requirement " + req.getName()
                                + " has no locations, delete?")) {
                    req.getParent().removeChild(req);
                    break;
                }
            }
            req.saveAttributes();
        }
        browser.deleteLocation(locationId);
        try {
            saveDocument();
            TreesTracker.getResourceByNode(doc).refreshLocal(
                    IResource.DEPTH_INFINITE, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void dispose() {
        db.removeListener(treeChangeListener);
        TreesTracker.getTracker().removeListener(dbStateListener);
        super.dispose();
    }

    public Requirement getTargetRequirement() {
        Requirement req = (Requirement) Activator.getDefault().getActiveNode(
                "Requirement");
        if (req != null && req.getTreeDB() == doc.getTreeDB()) {
            return req;
        }
        return null;
    }

    public void close(boolean save) {
        getSite().getPage().closeEditor(this, save);
    }

    @Override
    public Object getAdapter(@SuppressWarnings("rawtypes") Class required) {
        if (IContentOutlinePage.class.equals(required)) {
            if (reqMarkerOutlinePage == null) {
                reqMarkerOutlinePage = new ReqMarkerOutlinePage();
                reqMarkerOutlinePage.setInput(doc);
                reqMarkerOutlinePage.addSelectionChangedListener(locChangeListener);
            }
            return reqMarkerOutlinePage;
        }
        return super.getAdapter(required);
    }
}
