package com.unitesk.requality.eclipse.editors.browser;

import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.views.CNFContentProvider;
import com.unitesk.requality.nodetypes.Location;

public class ReqMarkerOutlinePage extends ContentOutlinePage {
    private Object input = null;

    @Override
    public void createControl(Composite parent) {
        super.createControl(parent);
        getTreeViewer().setContentProvider(new CNFContentProvider());
        getTreeViewer().setSorter(new ViewerSorter());
        getTreeViewer().setLabelProvider(new LabelProvider() {
            @Override
            public Image getImage(Object element) {
                if (element instanceof TreeNode) {
                    TreeNode node = (TreeNode) element;
                    if (node.getQualifiedId().equals("/")) {
                        return PlatformUI.getWorkbench().getSharedImages()
                                .getImage(IDE.SharedImages.IMG_OBJ_PROJECT);
                    }
                    return Activator.getDefault().getImageRegistry()
                            .get("nodetype." + node.getType());
                }
                return super.getImage(element);
            }
            
            @Override
            public String getText(Object element) {
                if (element instanceof Location) {
                    Location loc = (Location) element;
                    if(loc.getRequirements().length==0) {
                        return "unbind location("+loc.getUUId()+")";
                    }
                    // TODO: many to many relation support
                    return loc.getRequirements()[0].getName()+" ("+loc.getId()+")";
                }
                return super.getText(element);
            }
        });
        getTreeViewer().setInput(input);
    }
    
    public void setInput(Object input) {
        this.input = input;
    }

    public void refresh() {
        getTreeViewer().refresh();
    }
}
