/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.editors.panels;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Text;

public class SelectFilePanel extends AbstractPanel {
    private static final String BTN_BROWSE = "Browse...";

    private Text pathText = null;
    private String initalPath;
    private String curPath;

    private boolean onlyFolder = false;

    public void setPath(String path) {
        curPath = path;
        initalPath = path;
    }

    public SelectFilePanel(Composite parent, int style, String initalPath) {
        super(parent, style);
        this.initalPath = initalPath;
        this.curPath = initalPath;

        this.setLayout(new GridLayout(2, false));
        this.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        pathText = new Text(this, SWT.BORDER);
        GridData gdata = new GridData(GridData.FILL_HORIZONTAL);
        gdata.grabExcessHorizontalSpace = true;
        pathText.setLayoutData(gdata);
        pathText.setText(initalPath);

        Button rootBut = new Button(this, SWT.NONE);
        rootBut.setText(BTN_BROWSE);
        SelectionListener rootList = new SelectionListener() {
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
            }

            @Override
            public void widgetSelected(SelectionEvent e) {
                String path;
                if (onlyFolder) {
                    path = new DirectoryDialog(getShell()).open();
                } else {
                    path = new FileDialog(getShell()).open();
                }
                if (path != null) {
                    // System.out.println(path);
                    curPath = path;
                    pathText.setText(path);
                }
            }
        };
        rootBut.addSelectionListener(rootList);
        initModifyListener();
    }

    @Override
    protected void initModifyListener() {
        pathText.addListener(SWT.Modify, modifyListener);
    }

    @Override
    public boolean isChanged() {
        return !initalPath.equals(curPath);
    }

    public String getSelectedPath() {
        return pathText.getText();
    }

    public void setSelectOnlyFolder(boolean onlyFolder) {
        this.onlyFolder = onlyFolder;
    }
}
