package com.unitesk.requality.eclipse.views.actions.documents;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.views.actions.CNFAction;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;

public class CreateDocumentFolderAction extends CNFAction {
    private static final String CREATE_FOLDER = "Create Folder...";


    public CreateDocumentFolderAction(ISelectionProvider provider) {
        // TODO: 
//        super(provider);
        setText(CREATE_FOLDER);
        setToolTipText(CREATE_FOLDER);
        setImageDescriptor(PlatformUI.getWorkbench().getSharedImages()
                .getImageDescriptor(ISharedImages.IMG_TOOL_NEW_WIZARD));
    }

    @Override
    public boolean isEnabled() {
        if (getSelectedNode() == null) {
            return false;
        }
        TreeNode n = getSelectedNode();
        if (n instanceof Document) {
            return true;
        }
        if (n instanceof DocFolder) {
            return true;
        }
        return false;
    }

    @Override
    public void run() {
        TreeNode node = getSelectedNode();
        final DocFolder parent;
        if (node instanceof Document) {
            parent = (DocFolder) node.getParent();
        } else if (node instanceof DocFolder) {
            parent = (DocFolder) node;
        } else {
            throw new UnsupportedOperationException("Wrong selection type");
        }

        InputDialog dlg = new InputDialog(PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getShell(), "New document folder",
                "Enter folder name", "new_folder", new IInputValidator() {
                    @Override
                    public String isValid(String newText) {
                        if (newText.length() == 0) {
                            return "enter folder name";
                        }
                        if (newText.lastIndexOf("/") != -1) {
                            return "folder name can't contains '/'";
                        }
                        if (parent.findChild(newText) != null) {
                            return "child with same id is exist";
                        }
                        return null;
                    }
                });

        if (dlg.open() == Dialog.OK) {
            parent.createChild(dlg.getValue(), "DocFolder");
        }
    }
}
