package com.unitesk.requality.merge;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.unitesk.requality.merge.Location.InvalidLocationException;

public class MergeProcessor {

    private static final MergeProcessor instance = new MergeProcessor();
    private List<String> idsNotFound;
    private List<String> idsFound;
    private List<Location> locations;

    public int parentNull = 0;
    public int parentWrong = 0;
    public int spanNull = 0;
    public int textNull = 0;
    public int spanOrTextWrong = 0;
    public int contentWrong = 0;
    public int commonParentNull = 0;

    private MergeProcessor() {
	idsNotFound = new ArrayList<String>();
	idsFound = new ArrayList<String>();
	locations = new ArrayList<Location>();
    }

    public static MergeProcessor getInstance() {
	return instance;
    }

    public Document merge(Document v1, Document v2) {
	reset();
	Tree t1 = new Tree(v1);
	Tree t2 = new Tree(v2);
	locations = lookupLocations(t1);
	locations = updateLocations(locations);
	locations = traceLocations(locations, t2);
	Map<Node, List<Location>> locationMapping = mapLocations(locations);
	Map<Node, Fragment> fragmentMapping = makeFragments(locationMapping);
	Iterator<Node> i = fragmentMapping.keySet().iterator();
	while (i.hasNext()) {
	    Node anchorNode = i.next();
	    mergeFragment(fragmentMapping.get(anchorNode), anchorNode);
	    t2.updateTree();
	}
	return v2;
    }

    public void reset() {
	if (!idsNotFound.isEmpty())
	    idsNotFound.clear();
	if (!idsFound.isEmpty())
	    idsFound.clear();
	if (!locations.isEmpty())
	    locations.clear();

    }

    private Map<Node, Fragment> makeFragments(
	    Map<Node, List<Location>> locationMapping) {
	Map<Node, Fragment> mapping = new HashMap<Node, Fragment>();
	Iterator<Node> i = locationMapping.keySet().iterator();
	while (i.hasNext()) {
	    Node anchorNode = i.next();
	    List<Location> locations = locationMapping.get(anchorNode);
	    List<Node> relatedNodes = new ArrayList<Node>();
	    for (Location loc : locations) {
		List<Node> locNodes = loc.getRelatedNodes();
		for (Node x : locNodes)
		    if (!relatedNodes.contains(x))
			relatedNodes.add(x);
	    }
	    Fragment f = new Fragment();
	    for (Node x : relatedNodes)
		f.attachNode(x, true);
	    mapping.put(anchorNode, f);
	}
	return mapping;
    }

    private void mergeFragment(Fragment f, Node anchorNode) {
	org.w3c.dom.Node anchorDomNode = anchorNode.getDomNode();
	org.w3c.dom.Node fRoot = f.getRoot().getDomNode();
	fRoot = anchorDomNode.getOwnerDocument().importNode(fRoot, true);
	anchorDomNode.getParentNode().replaceChild(fRoot, anchorDomNode);
    }

    private Map<Node, List<Location>> mapLocations(List<Location> locations) {
	Map<Node, List<Location>> mapping = new HashMap<Node, List<Location>>();
	checkIds(locations);
	for (Location loc : locations) {
	    String id = loc.getLocationId();
	    Node anchorNode = loc.getAnchorNode();
	    if (anchorNode != null && !idsNotFound.contains(id)
		    && idsFound.contains(id)) {
		if (mapping.containsKey(anchorNode))
		    mapping.get(anchorNode).add(loc);
		else {
		    List<Location> list = new ArrayList<Location>();
		    list.add(loc);
		    mapping.put(anchorNode, list);
		}

	    }
	}
	return mapping;
    }

    private void checkIds(List<Location> locations) {
	for (Location loc : locations) {
	    Node anchorNode = loc.getAnchorNode();
	    String id = loc.getLocationId();
	    if (anchorNode == null) {
		if (!idsNotFound.contains(id))
		    idsNotFound.add(id);
		if (idsFound.contains(id))
		    idsFound.remove(id);
	    } else if (!idsNotFound.contains(id) && !idsFound.contains(id))
		idsFound.add(id);
	}
    }

    private List<Location> traceLocations(List<Location> locations, Tree t) {
	List<Location> newLocations = new ArrayList<Location>(locations);
	Collections.copy(newLocations, locations);
	for (Location loc : newLocations) {
	    Node newAnchorNode = traceLocation(loc, t);
	    loc.setAnchorNode(newAnchorNode);
	}
	return newLocations;
    }

    private Node traceLocation(Location loc, Tree t) {
	if(loc.getLocationId().equals("b27170bc-cfb7-4397-af9f-2041b9e5ce40")){
	    int stop=0;
	    stop++;
	}
	Node commonParent = loc.getCommonParent();
	NodeFilter cpFilter = new NodeFilter(commonParent, true, true, false,
		false);
	List<Node> commonParentMatches = t.nearestMatch(commonParent, cpFilter);
	for (Node commonParentMatch : commonParentMatches) {
	    NodeFilter parentFilter = new NodeFilter(loc.getSpanNode()
		    .getParent(), true, true, false, false);
	    List<Node> spanParents = t.nearestMatch(loc.getSpanNode()
		    .getParent(), commonParentMatch, parentFilter, true);
	    for (Node spanParent : spanParents) {
		if (spanParent != null
			&& spanParent.isEqual(loc.getSpanNode().getParent())) {
		    if (spanParent.isSame(commonParentMatch)
			    || spanParent.getContext().isWithin(
				    commonParentMatch.getContext())) {
			NodeFilter spanFilter = new NodeFilter(
				loc.getAnchorNode(), true, false, false, true);
			List<Node> spanNearestMatches = t.nearestMatch(
				loc.getSpanNode(), spanParent, spanFilter);
			if (!spanNearestMatches.isEmpty()) {
			    Node spanNearestMatch = spanNearestMatches.get(0);
			    NodeFilter textFilter = spanFilter;
			    List<Node> textNearestMatches = t
				    .nearestMatch(loc.getAnchorNode(),
					    spanParent, textFilter);
			    if (!textNearestMatches.isEmpty()) {
				Node textNearestMatch = textNearestMatches
					.get(0);
				if (spanNearestMatch != null
					&& textNearestMatch != null
					&& spanNearestMatch
						.equals(textNearestMatch)
					&& textNearestMatch.getDomNode()
						.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
				    String nodeValue = textNearestMatch
					    .getDomNode().getNodeValue().trim();
				    if (nodeValue.equals(loc.getLocationText()
					    .trim())) {
					textNearestMatch.getContext()
						.excludeFromSearch(true);
					return textNearestMatch;
				    } else if (nodeValue.contains(loc
					    .getLocationText().trim()))
					return textNearestMatch;
				    else
					this.contentWrong++;
				} else
				    this.spanOrTextWrong++;
			    } else
				this.textNull++;
			} else
			    this.spanNull++;
		    } else
			this.parentWrong++;
		} else
		    this.parentNull++;
	    } 
	} this.commonParentNull++;
	return null;
    }

    public void merge(Reader v1, Reader v2, Writer output) {
	try {
	    DocumentBuilderFactory factory = DocumentBuilderFactory
		    .newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    Document doc1 = builder.parse(new InputSource(v1));
	    Document doc2 = builder.parse(new InputSource(v2));
	    try {
		Document result = merge(doc1, doc2);
		doc2 = result;
	    } catch (NullPointerException e) {
		e.printStackTrace();
		throw e;
	    } finally {
		writeXmlFile(doc2, output);
	    }
	} catch (ParserConfigurationException e) {
	    e.printStackTrace();
	} catch (SAXException e) {
	    e.printStackTrace();
	} catch (IOException e) {
	    e.printStackTrace();
	}
    }

    private List<Location> lookupLocations(Tree t) {
	List<Location> locations = new ArrayList<Location>();
	for (Node x : t.getTraversalSequence()) {
	    if (Location.isLocationNode(x))
		try {
		    locations.add(new Location(x));
		} catch (InvalidLocationException e) {
		    e.printStackTrace();
		}
	}
	return locations;
    }
    
    private List<Location> updateLocations(List<Location> locations){
	List<Location> locs = new ArrayList<Location>(locations);
	Collections.copy(locs, locations);
	Map<String, Node> idToNode = new HashMap<String, Node>();
	for(Location loc: locs){
	    String id = loc.getLocationId();
	    if(id.equals("b27170bc-cfb7-4397-af9f-2041b9e5ce40")){
		    int stop=0;
		    stop++;
		}
	    if(!idToNode.containsKey(id)){
		List<Location> sameId = getLocationsById(locs, id);
		idToNode.put(id, getLocationCommonParent(sameId));
	    }
	    loc.setCommonParent(idToNode.get(id));
	}
	return locs;
    }
    
    private Node getLocationCommonParent(List<Location> sameIdLocations) {
	List<Node> locationSpanNodes = new ArrayList<Node>();
	for (Location loc : sameIdLocations)
	    locationSpanNodes.add(loc.getSpanNode());
	return Tree.findLCA(locationSpanNodes);
    }

    private static void writeXmlFile(Document doc, Writer output) {
	try {
	    Source source = new DOMSource(doc);

	    Result result = new StreamResult(output);

	    Transformer xformer = TransformerFactory.newInstance()
		    .newTransformer();
	    xformer.transform(source, result);
	} catch (TransformerConfigurationException e) {
	} catch (TransformerException e) {
	}
    }

    private List<Location> getLocationsById(List<Location> locations, String id) {
	List<Location> sameIdLocations = new ArrayList<Location>();
	for (Location loc : locations)
	    if (loc.getLocationId().equals(id))
		sameIdLocations.add(loc);
	return sameIdLocations;
    }

    private boolean isCommonParent(Location refLocation, Node parent) {
	List<Location> sameIdLocations = getLocationsById(locations,
		refLocation.getLocationId());
	if (!sameIdLocations.isEmpty() && sameIdLocations.size() != 1) {
	    Node commonParent = parent;
	    for (Location loc : sameIdLocations) {
		if (loc.equals(refLocation))
		    continue;
		if (loc.wasUpdated() && loc.getAnchorNode() != null) {
		    Node currentLocParent = loc.getAnchorNode().getParent();
		    if (commonParent.isSame(currentLocParent))
			continue;
		    if (currentLocParent.getContext().getDepth() > commonParent
			    .getContext().getDepth()) {
			if (currentLocParent.getContext().isWithin(
				commonParent.getContext()))
			    continue;
			else
			    return false;
		    } else if (currentLocParent.getContext().getDepth() < commonParent
			    .getContext().getDepth()) {
			if (commonParent.getContext().isWithin(
				currentLocParent.getContext())) {
			    commonParent = currentLocParent;
			    continue;
			} else
			    return false;
		    } else {
			if (currentLocParent.getParent().isSame(
				commonParent.getParent())) {
			    commonParent = commonParent.getParent();
			    continue;
			} else
			    return false;
		    }

		}
	    }
	}
	return true;
    }

    public List<String> getMissingIds() {
	return this.idsNotFound;
    }

    public List<String> getMergedIds() {
	return this.idsFound;
    }
}
