package com.unitesk.requality.merge;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Document;

public class Tree {

    private List<Node> levelOrderedTraversal;

    protected Tree() {
	levelOrderedTraversal = new ArrayList<Node>();
    }

    public Tree(org.w3c.dom.Node domNode) {
	makeLevelOrderedTraversalSequence(domNode);
    }

    public Tree(Document doc) {
	makeLevelOrderedTraversalSequence(doc.getDocumentElement());
    }

    protected void makeLevelOrderedTraversalSequence(org.w3c.dom.Node root) {
	levelOrderedTraversal = new ArrayList<Node>();
	levelOrderedTraversal.add(new Node(root));
	for (int i = 0; i < levelOrderedTraversal.size(); i++) {
	    Node currentNode = levelOrderedTraversal.get(i);
	    if (!currentNode.getDomNode().hasChildNodes())
		continue;
	    for (org.w3c.dom.Node x = currentNode.getDomNode().getFirstChild(); x
		    .getNextSibling() != null; x = x.getNextSibling())
		levelOrderedTraversal.add(new Node(currentNode, x));
	    levelOrderedTraversal.add(new Node(currentNode, currentNode
		    .getDomNode().getLastChild()));
	}
    }

    public Iterator<Node> traverse() {
	return levelOrderedTraversal.listIterator();
    }

    public List<Node> getTraversalSequence() {
	return levelOrderedTraversal;
    }

    public Node getRoot() {
	return levelOrderedTraversal.get(0);
    }

    public List<Node> nearestMatch(Node x) {
	return nearestMatch(x, this.levelOrderedTraversal);
    }

    public List<Node> nearestMatch(Node x, NodeFilter f) {
	return nearestMatch(x, f.filter(this.levelOrderedTraversal));
    }

    public List<Node> nearestMatch(Node x, Node parent) {
	return nearestMatch(x, parent.getChilds());
    }

    public List<Node> nearestMatch(Node x, Node parent, boolean deep) {
	if (deep)
	    return nearestMatch(x, parent.getDescendants());
	else
	    return nearestMatch(x, parent.getChilds());
    }

    public List<Node> nearestMatch(Node x, Node parent, NodeFilter f) {
	return nearestMatch(x, f.filter(parent.getChilds()));
    }

    public List<Node> nearestMatch(Node x, Node parent, NodeFilter f,
	    boolean deep) {
	if (deep)
	    return nearestMatch(x, f.filter(parent.getDescendants()));
	else
	    return nearestMatch(x, f.filter(parent.getChilds()));
    }

    private List<Node> nearestMatch(Node x, List<Node> possibleMatches) {
	if (possibleMatches.isEmpty())
	    return new ArrayList<Node>();
	final Context refContext = x.getContext();
	final Comparator<Node> POSITIONAL_ORDER = new Comparator<Node>() {
	    public int compare(Node x, Node y) {
		return x.getContext().positionDistance(refContext)
			- y.getContext().positionDistance(refContext);
	    }
	};
	final Comparator<Node> SIMILARITY_ORDER = Collections
		.reverseOrder(new Comparator<Node>() {
		    public int compare(Node x, Node y) {
			return x.getContext().contextSimilarity(refContext)
				- y.getContext().contextSimilarity(refContext);
		    }
		});
	Collections.sort(possibleMatches, POSITIONAL_ORDER);
	Collections.sort(possibleMatches, SIMILARITY_ORDER);
	if (possibleMatches.size() > 3)
	    return possibleMatches.subList(0, 2);
	return possibleMatches;
    }

    public boolean isEqual(Tree t) {
	if (this.levelOrderedTraversal.size() != t.levelOrderedTraversal.size())
	    return false;
	Iterator<Node> i = this.traverse();
	Iterator<Node> j = t.traverse();
	while (i.hasNext() && j.hasNext()) {
	    Node reference = i.next();
	    Node match = j.next();
	    if (!reference.isEqual(match))
		return false;
	}
	return true;
    }

    public void updateTree() {
	makeLevelOrderedTraversalSequence(getRoot().getDomNode());
    }

    public static Node findLCA(List<Node> nodes) {
	if (!nodes.isEmpty()) {
	    Node lca = nodes.get(0).getParent();
	    if (lca == null)
		lca = nodes.get(0);
	    for (Node curr : nodes) {
		Node currParent = curr.getParent();
		if (currParent != null && !lca.isSame(currParent)) {
		    Context currParentCtx = currParent.getContext();
		    if (!currParentCtx.isWithin(lca.getContext())) {
			if (!lca.getContext().isWithin(currParentCtx)) {
			    while (!currParentCtx.isWithin(lca.getContext())) {
				lca = lca.getParent();
			    }
			} else
			    lca = currParent;
		    }
		}

	    }
	    return lca;
	}
	return null;
    }
}
