package com.unitesk.requality.nodetypes;

import java.util.UUID;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;


public class NodeType extends TreeNode {
    private static final String ATTRIBUTE_CLASSNAME = "_classname";
    private static final String ATTRIBUTE_ICON = "_iconResource";
    private Class<? extends TreeNode> clazz;
    public NodeType(String name, TreeDB db) {
        super(db, UUID.randomUUID());
        this.id = name;
        setType("NodeType");
    }

    public NodeType(TreeDB db, UUID uuid) {
        super(db, uuid);
    }

    public String getName() {
        return getId();
    }

    @Override
    public String toString() {
        return "TypeNode '" + getName() + "' (" + getUUId() + ")";
    }

    public void setClassName(String className) {
        putAttribute(new Attribute(AttributeType.STRING, ATTRIBUTE_CLASSNAME, className));
    }

    public void setClass(Class<? extends TreeNode> clazz) {
        putAttribute(new Attribute(AttributeType.STRING, ATTRIBUTE_CLASSNAME, clazz.getName()));
        this.clazz=clazz;
    }

    @SuppressWarnings("unchecked")
    public Class<? extends TreeNode> getTypeClass() {
        if(clazz!=null) {
            return clazz;
        }
        String className = (String) getAttributeValue(ATTRIBUTE_CLASSNAME);
        if(className==null) {
            return null;
        }
        try {
            return (Class<? extends TreeNode>) Class.forName(className);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        return null;
    }

    public String getIconName() {
        if (getAttributeValue(ATTRIBUTE_ICON) == null)
            return getId();
        return (String) getAttributeValue(ATTRIBUTE_ICON);
    }

    public void setName(String icon) {
        putAttribute(new Attribute(AttributeType.STRING, ATTRIBUTE_ICON, icon));
    }

    @Override
    public boolean equals(Object obj) {
        if( super.equals(obj) ) return true;
        if(obj instanceof NodeType) {
            NodeType t = (NodeType) obj;
            if(t.getName().equals(getName())) return true;
        }
        return false;
    }
}
