package com.unitesk.requality.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.natures.ReqCatalogue;
import com.unitesk.requality.eclipse.tools.ResourceStorage;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.xml.diff.DiffXml;

public class DiffXMLTraceLocationsTest {
    private IProject project;

    private TreeDB getDB() {
        return TreesTracker.getTracker().getTree(project);
    }
    
    @Before
    public void setUp() {
        try {
            project = ResourcesPlugin.getWorkspace().getRoot()
                    .getProject("TestProject");
            project.delete(true, null);
            project.create(null);
            project.open(null);

            IProjectDescription description = project.getDescription();
            String[] natures = description.getNatureIds();
            String[] newNatures = new String[natures.length + 1];
            System.arraycopy(natures, 0, newNatures, 0, natures.length);
            newNatures[natures.length] = ReqCatalogue.ID;
            description.setNatureIds(newNatures);
            project.setDescription(description, null);

            ResourceStorage rs = (ResourceStorage) getDB().getStorage();
            TreeNode root = getDB().getRootNode();
            Document docs = (Document) root.createChild("Documents", "Document");
            IFolder rootContainer = (IFolder) rs.getResource(docs.getUUId()).getParent();
            IPath location = Path.fromOSString(new File("test-data/sameDocumentXMLDiff").getAbsolutePath());
            IFolder docSrcFolder = rootContainer.getFolder(new Path("Documents.src"));
            try {
                docSrcFolder.createLink(location, IResource.NONE, null);
                IFolder docFolder = rootContainer.getFolder("Documents.src");
                docFolder.copy(rootContainer.getFullPath().append("Documents"), false, null);
            } catch (CoreException e) {
                fail("test data init fail");
            }

        } catch (CoreException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void sameDocument() {
        ResourceStorage rs = (ResourceStorage) getDB().getStorage();
        Document oldDoc = (Document) getDB().getNode("/Documents/old.xhtml");
        Document newDoc = (Document) getDB().getNode("/Documents/new.xhtml");
        String oldPath = rs.getResourceFolder(oldDoc.getUUId()).getFile(oldDoc.getAttributeValue("file").toString()).getLocation().toOSString();;
        String newPath = rs.getResourceFolder(newDoc.getUUId()).getFile(newDoc.getAttributeValue("file").toString()).getLocation().toOSString();;

        DiffXml.merge(oldPath, newPath, newPath);
        List<String> persistentLocations = DiffXml.getPersistentLocations();
        for(String id : persistentLocations){
            Location loc = (Location)newDoc.createChild(id, "Location");
            loc.setDescription(DiffXml.getLocationDescription(id));
            loc.saveAttributes();
        }
        
        assertEquals("4 locations in test data found", 4, newDoc.sizeChildren());
        assertEquals("4 locations in test data transported", 4, newDoc.getLocations().size());

        // Check nodes
        for(int i = 0; i < oldDoc.sizeChildren(); i++) {
            Location oldLoc = (Location) oldDoc.getChild(i);
            String oldLocId = oldLoc.getId();
            String oldLocDesc = oldLoc.getDescription();
            boolean found = false;
            for(int j = 0; j < newDoc.sizeChildren(); j++) {
                //if(((Location) newDoc.getChild(j)).getDescription().equals(oldLocDesc)) {
        	if(((Location) newDoc.getChild(j)).getDescription().trim().equals(oldLocDesc.trim())) { //temp fix to mitigate issue #404
                    found = true;
                    break;
                }
            }
            assertTrue("location '" + oldLocId + "' not exist", found);
        }

        // Check new document
        for(int i = 0; i < oldDoc.sizeChildren(); i++) {
            Location oldLoc = (Location) oldDoc.getChild(i);
            String oldLocId = oldLoc.getId();
            String oldLocDesc = oldLoc.getDescription();
            boolean found = false;
            for(UUID locUUID : newDoc.getLocations()) {
                //if(newDoc.getLocation(locUUID).getDescription().equals(oldLocDesc)) {
        	if(newDoc.getLocation(locUUID.toString()).getDescription().trim().equals(oldLocDesc.trim())) { //temp fix to mitigate issue #404
                    found=true;
                    break;
                }
            }
            assertTrue("location '" + oldLocId + "' not exist in new document", found);
        }
    }
    
    @After
    public void tearDown(){
        try {
            project.delete(false, null);
        } catch (CoreException e) {
            e.printStackTrace();
        }
    }
}
