/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.tests.core;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.junit.Before;
import org.junit.Test;

import com.unitesk.requality.core.FSStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.tests.utils.Ulils;

public class DocumentExtractLocationText {
    private Document doc;

    String locUUID = "78885e7c-9d0c-48fa-a016-0f2801bc5017";
    public String getXhtlpWithLocationText(String text) {
        StringBuffer sb = new StringBuffer();
        sb.append("<html xmlns=\"http://www.w3.org/1999/xhtml\"><head><title>Sample.html</title></head>");
        sb.append("<body>");
        sb.append("<span class=\"requality_text id_"+locUUID+"\">");
        sb.append("<a name=\""+locUUID+"\" id=\"id_"+locUUID+"\" class=\"requality_id\"/>");
        sb.append(text);
        sb.append("</span>");
        sb.append("</body></html>");
        return sb.toString();
    }

    @Before
    public void setUp() throws IOException {
        String path = System.getProperty("java.io.tmpdir") + File.separator
                + "treedb.test";
        if (new File(path).exists()) {
            Ulils.recursiveDelete(new File(path));
        }
        TreeDB db = new TreeDB(new FSStorage(path));
        db.addNodeType("Document", Document.class);
        TreeNode docRoot = db.getRootNode().createChild("Documents", "Document");
        doc = (Document) docRoot.createChild("test.xhtml");
        new File(doc.getHTMLPath()).getParentFile().mkdirs();
        new File(doc.getHTMLPath()).createNewFile();
        FileWriter fw = new FileWriter(doc.getHTMLPath());
        fw.write(getXhtlpWithLocationText("Hello, World"));
        fw.close();
    }

    @Test
    public void locationTextExtraction() {
        String locationText = doc.extractLocationText(locUUID);
        assertEquals("Hello, World", locationText);
        
    }
}
