package patterns;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import org.eclipse.swtbot.eclipse.finder.SWTWorkbenchBot;
import org.eclipse.swtbot.swt.finder.junit.SWTBotJunit4ClassRunner;
import org.eclipse.swtbot.swt.finder.widgets.SWTBotShell;
import org.eclipse.swtbot.swt.finder.widgets.SWTBotTree;
import org.eclipse.swtbot.swt.finder.widgets.SWTBotTreeItem;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;

import swtbotforrequality.ContextMenuHelper;

@RunWith(SWTBotJunit4ClassRunner.class)
public class ImportDoc {
	private static SWTWorkbenchBot	bot;
	private String docPath = "D:\\Work\\WorkEclipses\\eclipse32Target\\DocsForReqs\\";
	private String goodDocName = "10.html";
	private String badDocName = "100.html";
	private String goodDocUTFName = "BigReqDoc_utf8.html";
	private String goodDocWindowsName = "BigReqDoc_1251.html";

	@BeforeClass
	public static void beforeClass() throws Exception {
		// 
		
		bot = new SWTWorkbenchBot();
		bot.viewByTitle("Welcome").close();
		bot.menu("File").menu("New").menu("Other...").click();
		 
		SWTBotShell shell = bot.shell("New");
		shell.activate();
		bot.tree().expandNode("Requality").select("Requality Project");
		bot.button("Next >").click();
		bot.textWithLabel("Project name:").setText("ReqProject");
		bot.button("Finish").click();
		bot.activeView();
	}

//	@Test
//	public void useBrowseButton() throws Exception {
//		//Documents ->   -> Import Document -> ,    Browse
//		// Browse ->   -> OK
//		//,   Documents      xhtml.
//		
//		//!!!!SWTBot       native dialog.
//	}
	
	@Test
	public void useCopyPaste() throws Exception {
		//Documents ->   -> Import Document -> ,    Browse
		//  Import file  ()    -> OK
		//,   Documents      xhtml.
		SWTBotTree tree = bot.viewByTitle("Requality Explorer").bot().tree();
		SWTBotTreeItem t1 = tree.expandNode("ReqProject");
		SWTBotTreeItem treeItem = t1.select("Documents");
		ContextMenuHelper.clickContextMenu(tree, "Import Document...");
		bot.shell("Import").activate();
		bot.textWithLabel("Import file:");
		bot.textWithLabel("Import file:").setText(docPath + goodDocName);
		bot.button("OK").click();
		assertTrue("Document 10.xhtml doesn't exists", tree.expandNode("ReqProject").expandNode("Documents").getNodes().contains("10.xhtml"));
	}
	
//	@Test
//	public void useCopyPasteWrongFileName() throws Exception {
//		//Documents ->   -> Import Document -> ,    Browse
//		//  Import file  ()     -> OK
//		//,     ,     .
//		
//		SWTBotTree tree = bot.viewByTitle("Requality Explorer").bot().tree();
//		SWTBotTreeItem t1 = tree.expandNode("ReqProject");
//		SWTBotTreeItem treeItem = t1.select("Documents");
//		ContextMenuHelper.clickContextMenu(tree, "Import Document...");
//		bot.shell("Import").activate();
//		bot.textWithLabel("Import file:");
//		bot.textWithLabel("Import file:").setText("D:\\Work\\WorkEclipses\\eclipse32Target\\100.html");
//		//   
//		bot.button("OK").click();
//	
//	//!!!!SWTBot       native dialog.
//	}
	
	@Test
	public void addUTF() throws Exception {
		//Documents ->   -> Import Document -> ,    Browse
		// Browse ->     UTF-8
		//,    Encoding   utf-8 -> OK
		//,   Documents      xhtml.
		SWTBotTree tree = bot.viewByTitle("Requality Explorer").bot().tree();
		SWTBotTreeItem t1 = tree.expandNode("ReqProject");
		SWTBotTreeItem treeItem = t1.select("Documents");
		ContextMenuHelper.clickContextMenu(tree, "Import Document...");
		bot.shell("Import").activate();
		bot.textWithLabel("Import file:");
		bot.textWithLabel("Import file:").setText(docPath + goodDocUTFName);
		assertEquals(bot.comboBoxWithLabel("Encoding:").getText(), "UTF-8");
		bot.button("OK").click();
		assertTrue("Document 10.xhtml doesn't exists", tree.expandNode("ReqProject").expandNode("Documents").getNodes().contains("10.xhtml"));
	}
	
	@Test
	public void addWindows() throws Exception {
		//Documents ->   -> Import Document -> ,    Browse
		// Browse ->     Windows1251
		//,    Encoding   Windows1251 -> OK
		//,   Documents      xhtml.
		SWTBotTree tree = bot.viewByTitle("Requality Explorer").bot().tree();
		SWTBotTreeItem t1 = tree.expandNode("ReqProject");
		SWTBotTreeItem treeItem = t1.select("Documents");
		ContextMenuHelper.clickContextMenu(tree, "Import Document...");
		bot.shell("Import").activate();
		bot.textWithLabel("Import file:");
		bot.textWithLabel("Import file:").setText(docPath + goodDocWindowsName);
		assertEquals(bot.comboBoxWithLabel("Encoding:").getText(), "windows-1251");
		bot.button("OK").click();
		assertTrue("Document 10.xhtml doesn't exists", tree.expandNode("ReqProject").expandNode("Documents").getNodes().contains("10.xhtml"));
	}
	
//	public void removeDoc() throws Exception {
//		//   ->   -> Remove Document
//		//,      
//		// OK-> ,   Documents   .
//	}
//	
//	@Test
//	public void addExistedDoc() throws Exception {
//		//Documents ->   -> Import Document
//		// Browse ->   -> OK
//		//,   Documents      xhtml.
//		//Documents ->   -> Import Document
//		// Browse ->     -> OK
//		//,     ,     
//		// .
//		//,     .
//	}
//	
//	@Test
//	public void updateDocCase1() throws Exception {
//		// 2   ,        ..
//		//     .
//		// Requality Explorer    ->   -> Update Document
//		//,   Document Selection     .
//		//,   OK .
//		//  Document Selection   . 
//		//,   OK  -> OK
//		//,        ,    1.
//		//,          .
//		// Show identifiers     ,   .
//
//	}
//	
//	@Test
//	public void updateDocCase2() throws Exception {
//		// 2   ,        ..
//		//     .
//		//      ,      .
//		// Requality Explorer    ->   -> Update Document
//		//  Document Selection   . -> OK
//		//,           ,    .
//		//,      .
//		//,          .
//		// Show identifiers     ,   .
//
//	}
//	
//	@Test
//	public void updateDocCase3() throws Exception {
//		// 2   .
//		//     .
//		// Requality Explorer    ->   -> Update Document
//		//  Document Selection   . -> OK
//		//,     ,     . -> OK
//		//,     Document Selection.
//
//	}
	
	@AfterClass
	public static void afterClass() throws Exception {
		bot.sleep(4000);
	}
}
